/*
 * Copyright (c) 2010 LAAS/CNRS                       --  Oct 2010
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
/*------------------------------------------------------------
 * 
 *                     --  Module  LWR  --
 * 
 *  Description:
 *  Creation date : Wed May 19 22:35:29 2010 
 *  Author: Xavier Broquere
 *  LAAS-CNRS, Toulouse FRANCE
 *------------------------------------------------------------*/
#ifndef LWRCONST_H
#define LWRCONST_H


#define LWR_DEBUG   1

#define LWR_PERIOD 0.01
#define LWR_TIC 1

#define JNT_TRQ_NB_TIC_MEAN 20

// FRI-related constants
#define MAX_REQ_CMD              600    /* i.e. 2 secs; try to reduce */
#define CMD_CHANNEL               15
#define ARG_CHANNEL               14

#define FRI_IDLE                   0
#define FRI_START                  1
#define FRI_STOP                   2
#define POWER_ON                   3
#define SET_TOOL                   4
#define SET_BASE                   5
#define RSET_STATUS                6
#define CART_STIFF                 7
#define CART_DAMP                  8
#define AXIS_STIFF                 9
#define AXIS_DAMP                 10
#define CART_CTRL                 11
#define AXIS_CTRL                 12
#define GRAV_COMP                 13
#define POSIT_CTRL                14
#define START_POSE                15
#define RSET_COUNT                16

#define FRI_OPEN                  88
#define FRI_CLOSE                 99
#define GOTO_Q_ABS                55
#define GOTO_Q_REL                56
#define GOTO_X_ABS                66
#define GOTO_X_REL                67


// Soft motion constants
#define LWR_SM_MAX_SAMPLES        10000
#define LWR_SM_CART_COORDS        6

// Speed limits for each joint
#define LWR_A1_MAX_VELOCITY       1.963495409   /* 112.5 degrees/sec  */
#define LWR_A2_MAX_VELOCITY       1.963495409   /* 112.5 degrees/sec  */
#define LWR_E1_MAX_VELOCITY       1.963495409   /* 112.5 degrees/sec  */
#define LWR_A3_MAX_VELOCITY       1.963495409   /* 112.5 degrees/sec  */
#define LWR_A4_MAX_VELOCITY       3.141592654   /* 180.0 degrees/sec  */
#define LWR_A5_MAX_VELOCITY       1.963495409   /* 112.5 degrees/sec  */
#define LWR_A6_MAX_VELOCITY       1.963495409   /* 112.5 degrees/sec  */

// Speed limits for cartesian movements
#define LWR_MAX_ANG_VELOCITY      0.418879021   /*  24.0 degrees/sec  */
#define LWR_MAX_LIN_VELOCITY      0.4           /* 400.0 mm/sec       */

#define LWR_DEF_X_STIFFNESS       5000    /* default X axis stiffness */
#define LWR_DEF_Y_STIFFNESS       5000    /* default Y axis stiffness */
#define LWR_DEF_Z_STIFFNESS       5000    /* default Z axis stiffness */
#define LWR_DEF_A_STIFFNESS        300    /* default A rot. stiffness */
#define LWR_DEF_B_STIFFNESS        300    /* default B rot. stiffness */
#define LWR_DEF_C_STIFFNESS        300    /* default C rot. stiffness */

#define LWR_DEF_X_DAMPING          0.7    /* default X axis damping   */
#define LWR_DEF_Y_DAMPING          0.7    /* default Y axis damping   */
#define LWR_DEF_Z_DAMPING          0.7    /* default Z axis damping   */
#define LWR_DEF_A_DAMPING          0.7    /* default A rot. damping   */
#define LWR_DEF_B_DAMPING          0.7    /* default B rot. damping   */
#define LWR_DEF_C_DAMPING          0.7    /* default C rot. damping   */

// Speed limit multiplicative factor
#define LWR_DEF_SPEED_LIMIT       0.2           /* 20% of max speed   */

// Rotation range for each joint
#define LWR_A1_RANGE_MIN        - 2.967059729   /* q1: -170.0 degrees */
#define LWR_A2_RANGE_MIN        - 2.094395103   /* q2: -120.0 degrees */
#define LWR_E1_RANGE_MIN        - 2.967059729   /* q3: -170.0 degrees */
#define LWR_A3_RANGE_MIN        - 2.094395103   /* q4: -120.0 degrees */
#define LWR_A4_RANGE_MIN        - 2.967059729   /* q5: -170.0 degrees */
#define LWR_A5_RANGE_MIN        - 2.268928028   /* q6: -130.0 degrees */
#define LWR_A6_RANGE_MIN        - 2.967059729   /* q7: -170.0 degrees */

#define LWR_A1_RANGE_MAX        + 2.967059729   /* q1: +170.0 degrees */
#define LWR_A2_RANGE_MAX        + 2.094395103   /* q2: +120.0 degrees */
#define LWR_E1_RANGE_MAX        + 2.967059729   /* q3: +170.0 degrees */
#define LWR_A3_RANGE_MAX        + 2.094395103   /* q4: +120.0 degrees */
#define LWR_A4_RANGE_MAX        + 2.967059729   /* q5: +170.0 degrees */
#define LWR_A5_RANGE_MAX        + 2.268928028   /* q6: +130.0 degrees */
#define LWR_A6_RANGE_MAX        + 2.967059729   /* q7: +170.0 degrees */

// MGD/MGI constants
#define LWR_MGI_EPSILON           0.0001
#define LWR_R1                    0.3105        /* BASE > R0 : 310.5 mm */
#define LWR_R3                    0.400         /* R2 > R3   : 400.0 mm */
#define LWR_R5                    0.390         /* R4 > R5   : 390.0 mm */
#define LWR_R7                    0.078         /* R7 > R8   :  78.0 mm */
#define LWR_R8_NO_TOOL            0.000         /* R8 > TOOL :   0.0 mm */
#define LWR_R8_GRIPPER            0.180         /* R8 > TOOL : 180.0 mm */
#define LWR_R8_DLR_HIT            0.130         /* R8 > TOOL : 130.0 mm */
#define LWR_GRIPPER_ROT           0.392699082   /* still not used  TODO */

// Other physical constants
#define LWR_CART_FRM_DIM          12  
#define LWR_CART_VEC              6             /* Cart vector elements */
#define LWR_JOINTS                7             /* LWR number of joints */
#define LWR_USER_SIZE             16  
#define LWR_JNT_SECURITY_MARGIN   0.008726646   /* 0.5 degrees */

#define LWR_FORCES_WINDOW         1024          /* The size of forces stored*/




  
// Used to define the size of the poster position for TrackQ:
// 100 tics/seconde * 1 minute
#define LWR_TRACK_MAX_SIZE        100*60*1

#endif
