/*
 * Copyright (c) 2010 LAAS/CNRS                       --  Oct 2010
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
/*------------------------------------------------------------
 * 
 *                     --  Module  LWR  --
 * 
 *  Description:
 *  Creation date : Wed May 19 22:35:29 2010 
 *  Author: Xavier Broquere
 *  LAAS-CNRS, Toulouse FRANCE
 *------------------------------------------------------------*/


#ifndef _LWRSTRUCT_H
#define _LWRSTRUCT_H

#include "lwrConst.h"

/*
 *  External tool and reference base configuration
 */

typedef enum LWR_TOOL_ENUM {
  LWR_NO_TOOL = 1,
  LWR_GRIPPER = 2,
  LWR_DLR_HIT = 3
} LWR_TOOL_ENUM;

typedef enum LWR_BASE_ENUM {
  LWR_NULL_BASE = 1,
  LWR_JIDO_BASE = 2
} LWR_BASE_ENUM;

typedef struct LWR_TOOL_CONFIG_STR {
  LWR_TOOL_ENUM  tool_type;
  double         distance;
} LWR_TOOL_CONFIG_STR;

typedef struct LWR_BASE_CONFIG_STR {
  LWR_BASE_ENUM  base_type;
} LWR_BASE_CONFIG_STR;


/*
 *  data for poster to export position, velocity, jacobian...
 */



typedef enum LWR_CNTRL_MODE_ENUM {
  LWR_OPEN_LOOP_TRACK_Q_CNTRL,
  LWR_OPEN_LOOP_CNTRL,
  LWR_TRAJECTORY_CNTRL,
  LWR_GOAL_CNTRL,
  LWR_EXTERNAL_POSITION_CNTRL,
  LWR_EXTERNAL_VELOCITY_CNTRL,
  LWR_FORCE_CNTRL
} LWR_CNTRL_MODE_ENUM ;

typedef struct LWR_POSITION_STR {
  Gb_q7 q;
  Gb_th th;
  Gb_q7 vq;
  Gb_q7 aq;
  Gb_q7 qold;
  Gb_q7 vqold;
  Gb_q7 aqold;
//  Gb_jac jac;
//  Gb_vitesse vw;
  double date;
} LWR_POSITION_STR;


typedef struct LWR_GOTO_CONF_STR {
  Gb_q7    confLeft;       // target configuration for the left arm
  Gb_q7    confRight;       // target configuration for the right arm
  GEN_BOOL wait_done;  // wait until motion is done
} LWR_GOTO_CONF_STR;

typedef struct LWR_GOTO_POSE_STR {
  Gb_dep   poseLeft;       // target position/rotation for the left arm
  Gb_dep   poseRight;      // target position/rotation for the right arm
  GEN_BOOL wait_done;      // wait until motion is done
} LWR_GOTO_POSE_STR;


/*
 *  Data for poster to inform about the current status of the arm
 */

// Control scheme: clone of the FRI_CTRL enum from FRI library to avoid genom dependencies
typedef enum LWR_CTRL_SCHEME_ENUM {
  LWR_CTRL_POSITION  = 1,
  LWR_CTRL_CART_IMP  = 2,
  LWR_CTRL_JOINT_IMP = 3,
  LWR_CTRL_GRAV_COMP = 0
} LWR_CTRL_SCHEME_ENUM;

typedef struct LWR_STATUS_STR {
  GEN_BOOL             sahandUpdate;
  GEN_BOOL             collisionMode;
  GEN_BOOL             collisionEnvMode;
  GEN_STRING128        collisionEnvPosterName;
  GEN_BOOL             simulation;
  GENMANIP_STRING1024  status_str;   // Human-readable description of current robot status:
                                     // FRI state, comm. quality, command mode and configuration
} LWR_STATUS_STR;

typedef enum LWR_ARM_ID_ENUM {
  LWR_ARM_LEFT = 0,
  LWR_ARM_RIGHT = 1
}LWR_ARM_ID_ENUM; 

/*
 *  Current movement structures and types
 */

typedef enum LWR_MOTION_TYPE_ENUM {
  LWR_NOT_MOVE,
  LWR_POSITION,
  LWR_CARTESIAN
} LWR_MOTION_TYPE_ENUM;

typedef enum LWR_MOTION_STATE_ENUM {
  LWR_STOPPED,
  LWR_STARTED,
  LWR_MOVING,
  LWR_FINISHED,
  LWR_ABORTED
} LWR_MOTION_STATE_ENUM;

typedef enum LWR_MOTION_ERROR_ENUM {
  LWR_OK,
  LWR_NOT_INITIALIZED,
  LWR_WRONG_Q_VALUE,
  LWR_SOFTMOTION_ERROR,
  LWR_FRI_MONITOR_MODE,
  LWR_MGI_CALC_ERROR,
  LWR_MGI_SINGULARITY,
  LWR_JNT_OUT_OF_RANGE,
  LWR_COLLISION_RISK
} LWR_MOTION_ERROR_ENUM;


/*
 *  Tracking structures
 */

typedef enum LWR_TRACK_MODE_ENUM {
  LWR_TRACK_FILE,
  LWR_TRACK_POSTER
} LWR_TRACK_MODE_ENUM;

typedef struct LWR_TRACK_STR {
  LWR_ARM_ID_ENUM     arm_id;
  GEN_STRING128       posterName;
  LWR_TRACK_MODE_ENUM trackMode;
} LWR_TRACK_STR;


/*
 *  Other structures and types
 */

// Init parameters structure
typedef struct LWR_INIT_ARM_STR {
  GEN_BOOL       is_used;
  int            lwr_port;
  GEN_STRING32   lwr_host;
  LWR_TOOL_ENUM  tool_type;
  LWR_BASE_ENUM  base_type;
} LWR_INIT_ARM_STR;


typedef struct LWR_INIT_STR {
  LWR_INIT_ARM_STR armLeft;
  LWR_INIT_ARM_STR armRight;
  GEN_STRING128  world_path;
  GEN_STRING128  world_file;
} LWR_INIT_STR;

typedef struct LWR_ARM_INST {
  GEN_BOOL       is_used;  
  LWR_POSITION_STR pos;    /* joint state / articulations */
  Gb_q7    currConf;
  Gb_dep   currPose;
  Gb_th th_7_tool;
  Gb_th th_8_tool;
  Gb_th th_base_0;
  Gb_v6 cartStiff;
  Gb_v6 cartDamp;
  float currentCartPos[LWR_CART_FRM_DIM];
  float currentJntVals[LWR_JOINTS];
  float currentMsrJntVals[LWR_JOINTS];
  float currentCmdVelJntVals[LWR_JOINTS];
  float currentCmdAccJntVals[LWR_JOINTS];
  LWR_TOOL_CONFIG_STR toolCfg;
  LWR_BASE_CONFIG_STR baseCfg;
  GEN_BOOL             power_on;
  int                  fri_state;    // monitor or command mode
  int                  fri_quality;  // communication quality: unacceptable, bad, ok or perfect
  LWR_CTRL_SCHEME_ENUM fri_scheme;   // command scheme: positions, stiffness, damping, torques
  LWR_CNTRL_MODE_ENUM cntrl_mode;    // control mode in the lwr genom module
  GEN_BOOL isStabilized;
  /* date to compute the acceleration and velocity numerically */
  double s_date0_tv_sec;
  double s_date0_tv_usec;
  double date1;
  double date2;
  float estExtTcpFT[LWR_CART_VEC];
  double estExtTcpFTHistory[LWR_FORCES_WINDOW][LWR_CART_VEC];

  float estExtJntTrq[LWR_JOINTS];
  float estMeanExtJntTrq[LWR_JOINTS]; 
  double estExtJntTrqHistory[LWR_JOINTS][JNT_TRQ_NB_TIC_MEAN];
  float msrJntTrq[LWR_JOINTS];
  double    desJntTrq[LWR_JOINTS];
  double    desTcpFT[LWR_CART_VEC];
  double timeStamp;
  int sock;
  int indexInTraj;
  GEN_BOOL isLeft;
  // the hand //
  GEN_BOOL isHandUsed;
  int indexHandInTraj;
  GENMANIP_POSITION_SAHAND_STR hand;
} LWR_ARM_INST;

typedef struct  LWR_CART_STIFF {
  LWR_ARM_ID_ENUM arm_id;
  Gb_v6 cartStiff;
} LWR_CART_STIFF;

typedef struct  LWR_CART_DAMP {
  LWR_ARM_ID_ENUM arm_id;
  Gb_v6 cartDamp;
} LWR_CART_DAMP;

typedef struct  LWR_FRI_SCHEME {
  LWR_ARM_ID_ENUM arm_id;
  LWR_CTRL_SCHEME_ENUM fri_scheme;
} LWR_FRI_SCHEME;
#endif
